#ifndef VOID_CONSOLE_CLASS
#define VOID_CONSOLE_CLASS

#include "I_console.h"

/*
==========================================
Console Command
==========================================
*/
struct CCommand
{
	CCommand(const char * iname, HCMD iid, I_ConHandler * ihandler)
	{
		name = new char[strlen(iname)+1];
		strcpy(name,iname);
		handler = ihandler;
		id = iid;
	}

	CCommand(const CCommand &cmd)
	{
		name = new char[strlen(cmd.name)+1];
		strcpy(name,cmd.name);
		handler = cmd.handler;
		id = cmd.id;
	}

	~CCommand()
	{	delete [] name;
		handler = 0;
	}

	HCMD	id;
	char *	name;
	I_ConHandler  * handler;
};


/*
==========================================
The main Console
==========================================
*/
class CConsole: public I_Console
				//public I_ConHandler
{
public:

	CConsole();
	~CConsole();

	//==============================================================
	//I_Console Interface
	void RegisterCVar(CVarBase * var,I_ConHandler * handler=0);
	void RegisterCommand(const char *cmdname,HCMD id,I_ConHandler * handler);
	void ComPrint(const char* text);
	bool ExecString(const char *string);

	//==============================================================
	//Command Handler
	void HandleCommand(HCMD cmdId, const CParms &parms);
	bool HandleCVar(const CVarBase * cvar, const CParms &parms) { return false; } 

	//==============================================================

	//commandline Parameters
	void AddCmdLineParm(const char * cmdLine);
	void ExecCmdLine();
	
	//Load a Config on start up which will contain 
	//archived values for cvars etc
	void LoadConfig(const char * szFilename);
	//just exec a config. might contain commands and cvar values
	void ExecConfig(const char * szFilename);
	//write cvars to given file
	void WriteCVars(const char * szFilename);

private:

	//==============================================================
	typedef std::list<CCommand>	 CmdList;
	typedef std::list<CVarBase*> CVarList;
	
	//List of registered commands
	CmdList		m_lCmds;		
	
	//List of registered cvars
	CVarList	m_lCVars;		

	//Hold parsed parms of commandString enterered
	CParms		m_parms;		
	
	//Current String in Console
	std::string	m_conString;	

	//Fixed List of previously entered strings
	StringList	m_cmdBuffer;	
	//iterator to keep track of the Command buffer
	StringList::iterator	m_itCmd;

	StringList  m_configFileParms;
	StringList  m_cmdLineParms;
	
	HANDLE		m_hLogFile;	

	//==============================================================
	void CVarlist (const CParms &parms);
	void CCmdList (const CParms &parms);
	void CFunctest(const CParms &parms);

	bool GetTokenParms(const char * token, CParms * parms);
	int  ReadConfigParm(char *buf, int bufsize, FILE * fp);
	bool IsCmdLineParm(const char * token, int tokenLen);

};

#endif